/**
 * @packageDocumentation
 * React components/hooks/utils used to easily connect to {@link @af-utils/virtual-core#VirtualScroller}
 */

/// <reference types="react" />

import type { ComponentProps } from 'react';
import type { ElementType } from 'react';
import { JSX as JSX_2 } from 'react/jsx-runtime';
import { ReactNode } from 'react';
import type { ScrollElement } from '@af-utils/virtual-core';
import { VirtualScroller } from '@af-utils/virtual-core';
import { VirtualScrollerEvent } from '@af-utils/virtual-core';
import { VirtualScrollerInitialParams } from '@af-utils/virtual-core';

/**
 * @public
 * React component.
 * {@link ListProps}.
 * Small abstraction, which in 90% cases allows to avoid hook boilerplate.
 */
export declare const List: <T extends ElementType = "div">({ model, children: Item, itemData, component, header, footer, getKey, tabIndex, style, ...props }: ListProps<T>) => JSX_2.Element;

/**
 * @public
 * Props passed to List item
 */
export declare type ListItemProps = {
    model: VirtualScroller;
    /** index */
    i: number;
    data?: any;
    offset?: number;
};

/**
 * @public
 * {@link List} component props
 */
export declare type ListProps<T extends ElementType = "div"> = Omit<ComponentProps<T>, "children" | "ref"> & {
    model: VirtualScroller;
    children: ElementType<ListItemProps>;
    itemData?: ListItemProps["data"];
    getKey?: (index: number, itemData: ListItemProps["data"]) => string | number;
    component?: T;
    header?: JSX.Element | null;
    footer?: JSX.Element | null;
};

/**
 * @public
 * Map current visible items
 */
export declare const mapVisibleRange: (model: VirtualScroller, cb: (index: number) => JSX.Element) => JSX.Element[];

/**
 * @public
 * Map current visible items and provide offset for each item.
 * Used in scenarios, when each item is absolutely positioned
 */
export declare const mapVisibleRangeWithOffset: (model: VirtualScroller, cb: (index: number, offset: number) => JSX.Element) => JSX.Element[];

/**
 * @public
 * React component.
 * Rerenders only on certain {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)}.
 * Allows to optimize performance.
 */
export declare const Subscription: (props: SubscriptionProps) => ReactNode;

/**
 * @public
 * {@link Subscription} component props
 */
export declare type SubscriptionProps = {
    model: VirtualScroller;
    children: () => ReactNode;
    events?: readonly VirtualScrollerEvent[] | VirtualScrollerEvent[];
};

/**
 * @public
 * React hook.
 * Rerenders component when one of {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} gets emitted.
 * Usually {@link Subscription} is a better alternative
 */
export declare const useComponentSubscription: (model: VirtualScroller, events?: readonly VirtualScrollerEvent[] | VirtualScrollerEvent[]) => void;

/**
 * @public
 * React hook.
 * Synchronizes scroller with model. Should be used in window-scroll cases, otherwise `ref={model.setScroller}` is preferrable.
 */
export declare const useScroller: (model: VirtualScroller, scroller: ScrollElement | null) => void;

/**
 * @public
 * React hook.
 * Allows to subscribe to {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} without unnecessary rerenders.
 * For example can be used in load-on-demand.
 */
export declare const useSubscription: (model: VirtualScroller, events: readonly VirtualScrollerEvent[] | VirtualScrollerEvent[], callBack: () => void) => void;

/**
 * @public
 * React hook.
 * Optimal CSS markup for virtual scroll is not intuitive.
 * Use this hook to avoid unneeded boilerplate.
 * @returns Array of 2 callback refs: [ outer, inner ].
 */
export declare const useSyncedStyles: (model: VirtualScroller) => [(el: HTMLElement | null) => void, (el: HTMLElement | null) => void];

/**
 * @public
 * React hook.
 * Calls {@link useVirtualModel} and synchronizes it with props
 * @param params - {@link @af-utils/virtual-core#VirtualScrollerInitialParams}
 * @returns same {@link @af-utils/virtual-core#VirtualScroller} instance
 */
export declare const useVirtual: (params: VirtualScrollerInitialParams) => VirtualScroller;

/**
 * @public
 * React hook.
 * Creates and stores one {@link @af-utils/virtual-core#VirtualScroller} instance.
 * It does not get recreated during component lifecycle.
 * Normally {@link useVirtual} should be used.
 * @param params - {@link @af-utils/virtual-core#VirtualScrollerInitialParams}
 * @returns same {@link @af-utils/virtual-core#VirtualScroller} instance
 */
export declare const useVirtualModel: (params: VirtualScrollerInitialParams) => VirtualScroller;

export { }
